from unittest import TestCase

import pytest
from django.contrib.auth.models import User
from testfixtures import OutputCapture, Replacer
from testfixtures.compat import PY3
from .models import SampleModel
from testfixtures.tests.test_django.manage import main

from ..test_compare import CompareHelper
from ... import compare
from ...django import compare as django_compare


class CompareTests(CompareHelper, TestCase):

    def test_simple_same(self):
        django_compare(SampleModel(id=1), SampleModel(id=1))

    def test_simple_diff(self):
        if PY3:
            expected = "'id': 1 != 2"
        else:
            expected = "u'id': 1 != 2"

        self.check_raises(
            SampleModel(id=1), SampleModel(id=2),
            compare=django_compare,
            message=(
                'SampleModel not as expected:\n'
                '\n'
                'same:\n'
                "['value']\n"
                '\n'
                'values differ:\n'+
                expected
            )
        )

    def test_simple_ignore_fields(self):
        django_compare(SampleModel(id=1), SampleModel(id=1),
                       ignore_fields=['id'])

    def test_ignored_because_speshul(self):
        django_compare(SampleModel(not_editable=1), SampleModel(not_editable=2))

    def test_ignored_because_no_longer_speshul(self):
        if PY3:
            same = "['created', 'id', 'value']\n"
        else:
            same = "['created', u'id', 'value']\n"
        self.check_raises(
            SampleModel(not_editable=1), SampleModel(not_editable=2),
            compare=django_compare,
            message=(
                'SampleModel not as expected:\n'
                '\n'
                'same:\n'+
                same+
                '\n'
                'values differ:\n'
                "'not_editable': 1 != 2"
            ),
            non_editable_fields=True
        )

    def test_normal_compare_id_same(self):
        # other diffs ignored
        compare(SampleModel(id=1, value=1), SampleModel(id=1, value=2))

    def test_normal_compare_id_diff(self):
        if PY3:
            expected = (
                "'id': 3 != 4\n"
                "'value': 1 != 2"
            )

        else:
            expected = (
                "'value': 1 != 2\n"
                "u'id': 3 != 4"
            )

        self.check_raises(
            SampleModel(id=3, value=1), SampleModel(id=4, value=2),
            compare=django_compare,
            message=(
                'SampleModel not as expected:\n'
                '\n'
                'values differ:\n'+
                expected
            )
        )

    def test_manage(self):
        with OutputCapture() as output:
            with Replacer() as r:
                r.replace('os.environ.DJANGO_SETTINGS_MODULE', '', strict=False)
                r.replace('sys.argv', ['x', 'check'])
                main()
        output.compare('System check identified no issues (0 silenced).')

    @pytest.mark.django_db
    def test_many_to_many_same(self):
        user = User.objects.create(username='foo')
        django_compare(user,
                       expected=User(
                           username='foo', first_name='', last_name='',
                           is_superuser=False
                       ),
                       ignore_fields=['id', 'date_joined'])



from itertools import chain
from subprocess import Popen as Popen, STDOUT, PIPE
from tempfile import TemporaryFile
from testfixtures.compat import basestring, PY3, zip_longest
from testfixtures.utils import extend_docstring

try:
    from unittest.mock import Mock
except ImportError:
    from mock import Mock


class MockPopen(object):
    """
    A specialised mock for testing use of :class:`subprocess.Popen`.
    An instance of this class can be used in place of the
    :class:`subprocess.Popen` and is often inserted where it's needed using
    :func:`mock.patch` or a :class:`Replacer`.
    """

    default_command = None

    def __init__(self):
        self.commands = {}
        self.mock = mock = Mock()
        self.mock.Popen.side_effect = self.Popen
        mock.Popen_instance = Mock(spec=Popen)
        inst = mock.Popen.return_value = mock.Popen_instance
        inst.communicate.side_effect = self.communicate
        inst.wait.side_effect = self.wait
        inst.send_signal.side_effect = self.send_signal
        inst.terminate.side_effect = self.terminate
        inst.kill.side_effect = self.kill
        inst.poll.side_effect = self.poll
        if PY3:
            def __enter__(self):
                return inst
            inst.__enter__ = __enter__

            def __exit__(self, exc_type, exc_val, exc_tb):
                inst.wait()
                for stream in inst.stdout, inst.stderr:
                    stream.close()

            inst.__exit__ = __exit__

    def set_command(self, command, stdout=b'', stderr=b'', returncode=0,
                    pid=1234, poll_count=3):
        """
        Set the behaviour of this mock when it is used to simulate the
        specified command.

        :param command: A string representing the command to be simulated.
        """
        self.commands[command] = (stdout, stderr, returncode, pid, poll_count)

    def set_default(self, stdout=b'', stderr=b'', returncode=0,
                    pid=1234, poll_count=3):
        """
        Set the behaviour of this mock when it is used to simulate commands
        that have no explicit behavior specified using
        :meth:`~MockPopen.set_command`.
        """
        self.default_command = (stdout, stderr, returncode, pid, poll_count)

    def __call__(self, *args, **kw):
        return self.mock.Popen(*args, **kw)

    def Popen(self, args, bufsize=0, executable=None,
              stdin=None, stdout=None, stderr=None,
              preexec_fn=None, close_fds=False, shell=False, cwd=None,
              env=None, universal_newlines=False,
              startupinfo=None, creationflags=0, restore_signals=True,
              start_new_session=False, pass_fds=(), encoding=None, errors=None):

        if isinstance(args, basestring):
            cmd = args
        else:
            cmd = ' '.join(args)

        behaviour = self.commands.get(cmd, self.default_command)
        if behaviour is None:
            raise KeyError('Nothing specified for command %r' % cmd)

        stdout_value, stderr_value, self.returncode, pid, poll = behaviour

        if stderr == STDOUT:
            line_iterator = chain.from_iterable(zip_longest(
                stdout_value.splitlines(True),
                stderr_value.splitlines(True)
            ))
            stdout_value = b''.join(l for l in line_iterator if l)
            stderr_value = None

        self.poll_count = poll
        for name, option, mock_value in (
            ('stdout', stdout, stdout_value),
            ('stderr', stderr, stderr_value)
        ):
            value = None
            if option is PIPE:
                value = TemporaryFile()
                value.write(mock_value)
                value.flush()
                value.seek(0)
            setattr(self.mock.Popen_instance, name, value)

        self.mock.Popen_instance.pid = pid
        self.mock.Popen_instance.returncode = None

        return self.mock.Popen_instance

    def wait(self):
        "Simulate calls to :meth:`subprocess.Popen.wait`"
        self.mock.Popen_instance.returncode = self.returncode
        return self.returncode

    def communicate(self, input=None):
        "Simulate calls to :meth:`subprocess.Popen.communicate`"
        self.wait()
        i = self.mock.Popen_instance
        return (i.stdout and i.stdout.read(),
                i.stderr and i.stderr.read())

    def poll(self):
        "Simulate calls to :meth:`subprocess.Popen.poll`"
        while self.poll_count and self.mock.Popen_instance.returncode is None:
            self.poll_count -= 1
            return None
        # This call to wait() is NOT how poll() behaves in reality.
        # poll() NEVER sets the returncode.
        # The returncode is *only* ever set by process completion.
        # The following is an artifact of the fixture's implementation.
        return self.wait()

    # These are here to check parameter types
    def send_signal(self, signal):
        "Simulate calls to :meth:`subprocess.Popen.send_signal`"
        pass

    def terminate(self):
        "Simulate calls to :meth:`subprocess.Popen.terminate`"
        pass

    def kill(self):
        "Simulate calls to :meth:`subprocess.Popen.kill`"
        pass


set_command_params = """
:param stdout:
    A string representing the simulated content written by the process
    to the stdout pipe.
:param stderr:
    A string representing the simulated content written by the process
    to the stderr pipe.
:param returncode:
    An integer representing the return code of the simulated process.
:param pid:
    An integer representing the process identifier of the simulated
    process. This is useful if you have code the prints out the pids
    of running processes.
:param poll_count:
    Specifies the number of times :meth:`MockPopen.poll` can be
    called before :attr:`MockPopen.returncode` is set and returned
    by :meth:`MockPopen.poll`.
"""


# add the param docs, so we only have one copy of them!
extend_docstring(set_command_params,
                 [MockPopen.set_command, MockPopen.set_default])



from testfixtures import Comparison as C, TempDirectory, compare
from testfixtures.compat import PY2, PY3, exception_module
from testfixtures.tests.sample1 import SampleClassA, a_function
from unittest import TestCase
import sys


class AClass:

    def __init__(self, x, y=None):
        self.x = x
        if y:
            self.y = y

    def __repr__(self):
        return '<'+self.__class__.__name__+'>'


class BClass(AClass):
    pass


class WeirdException(Exception):
    def __init__(self, x, y):
        self.x = x
        self.y = y


class X(object):
    __slots__ = ['x']

    def __repr__(self):
        return '<X>'


class FussyDefineComparison(object):

    def __init__(self, attr):
        self.attr = attr

    def __eq__(self, other):
        if not isinstance(other, self.__class__):  # pragma: no cover
            raise TypeError()
        return False  # pragma: no cover

    def __ne__(self, other):
        return not self == other  # pragma: no cover


class TestC(TestCase):

    def test_example(self):
        # In this pattern, we want to check a sequence is
        # of the correct type and order.
        r = a_function()
        self.assertEqual(r, (
            C('testfixtures.tests.sample1.SampleClassA'),
            C('testfixtures.tests.sample1.SampleClassB'),
            C('testfixtures.tests.sample1.SampleClassA'),
            ))
        # We also want to check specific parts of some
        # of the returned objects' attributes
        self.assertEqual(r[0].args[0], 1)
        self.assertEqual(r[1].args[0], 2)
        self.assertEqual(r[2].args[0], 3)

    def test_example_with_object(self):
        # Here we see compare an object with a Comparison
        # based on an object of the same type and with the
        # same attributes:
        self.assertEqual(
            C(AClass(1, 2)),
            AClass(1, 2),
            )
        # ...even though the original class doesn't support
        # meaningful comparison:
        self.assertNotEqual(
            AClass(1, 2),
            AClass(1, 2),
            )

    def test_example_with_vars(self):
        # Here we use a Comparison to make sure both the
        # type and attributes of an object are correct.
        self.assertEqual(
            C('testfixtures.tests.test_comparison.AClass',
              x=1, y=2),
            AClass(1, 2),
            )

    def test_example_with_odd_vars(self):
        # If the variable names class with parameters to the
        # Comparison constructor, they can be specified in a
        # dict:
        self.assertEqual(
            C('testfixtures.tests.test_comparison.AClass',
              {'x': 1, 'y': 2}),
            AClass(1, 2),
            )

    def test_example_not_strict(self):
        # Here, we only care about the 'x' attribute of
        # the AClass object, so we turn strict mode off.
        # With strict mode off, only attributes specified
        # in the Comparison object will be checked, and
        # any others will be ignored.
        self.assertEqual(
            C('testfixtures.tests.test_comparison.AClass',
              x=1,
              strict=False),
            AClass(1, 2),
            )

    def test_example_dont_use_c_wrappers_on_both_sides(self):
        # NB: don't use C wrappers on both sides!
        e = ValueError('some message')
        try:
            self.assertEqual(C(e), C(e))
        except Exception as e:
            self.failUnless(isinstance(e, AssertionError))
            self.assertEqual(
                e.args,
                (("<C(failed):{mod}.ValueError>wrong type</C> != \n"
                  "  <C:{mod}.ValueError>\n"
                  "  args:('some message',)\n"
                  "  </C>"
                  ).format(mod=exception_module), ))
        else:
            self.fail('No exception raised!')

    def test_repr_module(self):
        self.assertEqual(repr(C('datetime')), '<C:datetime>')

    def test_repr_class(self):
        self.assertEqual(
            repr(C('testfixtures.tests.sample1.SampleClassA')),
            '<C:testfixtures.tests.sample1.SampleClassA>'
            )

    def test_repr_function(self):
        self.assertEqual(
            repr(C('testfixtures.tests.sample1.z')),
            '<C:testfixtures.tests.sample1.z>'
            )

    def test_repr_instance(self):
        self.assertEqual(
            repr(C(SampleClassA('something'))),
            "\n"
            "  <C:testfixtures.tests.sample1.SampleClassA>\n"
            "  args:('something',)\n"
            "  </C>"
            )

    def test_repr_exception(self):
        self.assertEqual(
            repr(C(ValueError('something'))),
            ("\n"
             "  <C:{0}.ValueError>\n"
             "  args:('something',)\n"
             "  </C>"
             ).format(exception_module))

    def test_repr_exception_not_args(self):
        r = repr(C(WeirdException(1, 2)))

        if sys.version_info >= (3, 2, 4):
            # in PY3, even args that aren't set still appear to be there
            args = "  args:(1, 2)\n"
        else:
            args = "  args:()\n"

        self.assertEqual(
            r,
            "\n"
            "  <C:testfixtures.tests.test_comparison.WeirdException>\n"
            + args +
            "  x:1\n"
            "  y:2\n"
            "  </C>"
            )

    def test_repr_class_and_vars(self):
        self.assertEqual(
            repr(C(SampleClassA, {'args': (1,)})),
            "\n"
            "  <C:testfixtures.tests.sample1.SampleClassA>\n"
            "  args:(1,)\n"
            "  </C>"
            )

    def test_repr_nested(self):
        self.assertEqual(
            repr(C(SampleClassA, y=C(AClass), z=C(BClass(1, 2)))),
            "\n"
            "  <C:testfixtures.tests.sample1.SampleClassA>\n"
            "  y:<C:testfixtures.tests.test_comparison.AClass>\n"
            "  z:\n"
            "    <C:testfixtures.tests.test_comparison.BClass>\n"
            "    x:1\n"
            "    y:2\n"
            "    </C>\n"
            "  </C>"
            )

    def test_repr_failed_wrong_class(self):
        try:
            self.assertEqual(
                C('testfixtures.tests.test_comparison.AClass',
                  x=1, y=2),
                BClass(1, 2)
                )
        except Exception as e:
            self.failUnless(isinstance(e, AssertionError))
            self.assertEqual(e.args, ((
                "<C(failed):testfixtures.tests.test_comparison.AClass>"
                "wrong type</C> != <BClass>"
                ), ))
        else:
            self.fail('No exception raised!')

    def test_repr_failed_all_reasons_in_one(self):
        if PY3:
            expected = (
                "\n  <C(failed):testfixtures.tests.test_com[79 chars] </C>"
                " != <AClass>",
                )
        else:
            expected = (
                "\n"
                "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
                "  x:1 not in Comparison\n"
                "  y:5 != 2\n"
                "  z:'missing' not in other\n"
                "  </C> != <AClass>",
                )
        try:
            self.assertEqual(
                C('testfixtures.tests.test_comparison.AClass',
                  y=5, z='missing'),
                AClass(1, 2)
                )
        except Exception as e:
            self.failUnless(isinstance(e, AssertionError))
            self.assertEqual(e.args, expected)
        else:
            self.fail('No exception raised!')

    def test_repr_failed_not_in_other(self):
        if PY3:
            expected = (
                "\n  <C(failed):testfixtures.tests.test_com[39 chars] </C>"
                " != <AClass>",
                )
        else:
            expected = (
                "\n"
                "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
                "  z:(3,) not in other\n"
                "  </C> != <AClass>",
                )
        # use single element tuple to check %
        try:
            self.assertEqual(
                C('testfixtures.tests.test_comparison.AClass',
                  x=1, y=2, z=(3, )),
                AClass(1, 2)
                )
        except Exception as e:
            self.failUnless(isinstance(e, AssertionError))
            self.assertEqual(e.args, expected)
        else:
            self.fail('No exception raised!')

    def test_repr_failed_not_in_self_strict(self):
        # use single element tuple to check %
        if PY3:
            expected = (
                "\n  <C(failed):testfixtures.tests.test_com[44 chars] </C>"
                " != <AClass>",
                )
        else:
            expected = (
                "\n"
                "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
                "  x:(1,) not in Comparison\n"
                "  </C> != <AClass>",
                )
        try:
            self.assertEqual(
                C('testfixtures.tests.test_comparison.AClass', y=2),
                AClass((1, ), 2)
                )
        except Exception as e:
            self.failUnless(isinstance(e, AssertionError))
            self.assertEqual(e.args, expected)
        else:
            self.fail('No exception raised!')

    def test_repr_failed_not_in_self_not_strict(self):
        if PY3:
            expected = (
                "\n  <C(failed):testfixtures.tests.test_com[39 chars] </C>"
                " != <AClass>",
                )
        else:
            expected = (
                "\n"
                "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
                "  z:(3,) not in other\n"
                "  </C> != <AClass>",
                )
        try:
            self.assertEqual(
                C('testfixtures.tests.test_comparison.AClass',
                  x=1, y=2, z=(3, )),
                AClass(1, 2)
                )
        except Exception as e:
            self.failUnless(isinstance(e, AssertionError))
            self.assertEqual(e.args, expected)
        else:
            self.fail('No exception raised!')

    def test_repr_failed_one_attribute_not_equal(self):
        # use single element tuple to check %
        try:
            self.assertEqual(
                C('testfixtures.tests.test_comparison.AClass', x=1, y=(2, )),
                AClass(1, (3, ))
                )
        except Exception as e:
            self.failUnless(isinstance(e, AssertionError))
            self.assertEqual(
                e.args,
                ("\n"
                 "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
                 "  y:(2,) != (3,)\n"
                 "  </C> != <AClass>",
                 ))
        else:
            self.fail('No exception raised!')

    def test_repr_failed_nested(self):
        left_side = [C(AClass, x=1, y=2),
                     C(BClass, x=C(AClass, x=1), y=C(AClass))]
        right_side = [AClass(1, 3), AClass(1, 3)]

        # do the comparison
        left_side == right_side

        self.assertEqual(
            "[\n"
            "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
            "  y:2 != 3\n"
            "  </C>, \n"
            "  <C:testfixtures.tests.test_comparison.BClass>\n"
            "  x:\n"
            "    <C:testfixtures.tests.test_comparison.AClass>\n"
            "    x:1\n"
            "    </C>\n"
            "  y:<C:testfixtures.tests.test_comparison.AClass>\n"
            "  </C>]",
            repr(left_side)
            )

        self.assertEqual(
            "[<AClass>, <AClass>]",
            repr(right_side)
            )

    def test_repr_failed_nested_failed(self):
        left_side = [C(AClass, x=1, y=2),
                     C(BClass,
                       x=C(AClass, x=1, strict=False),
                       y=C(AClass, z=2))]
        right_side = [AClass(1, 2),
                      BClass(AClass(1, 2), AClass(1, 2))]

        # do the comparison
        left_side == right_side

        self.assertEqual(
            "[\n"
            "  <C:testfixtures.tests.test_comparison.AClass>\n"
            "  x:1\n"
            "  y:2\n"
            "  </C>, \n"
            "  <C(failed):testfixtures.tests.test_comparison.BClass>\n"
            "  y:\n"
            "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
            "  x:1 not in Comparison\n"
            "  y:2 not in Comparison\n"
            "  z:2 not in other\n"
            "  </C> != <AClass>\n"
            "  </C>]",
            repr(left_side)
            )

        self.assertEqual(
            '[<AClass>, <BClass>]',
            repr(right_side)
            )

    def test_repr_failed_passed_failed(self):
        c = C('testfixtures.tests.test_comparison.AClass', x=1, y=2)

        try:
            self.assertEqual(c, AClass(1, 3))
        except Exception as e:
            self.failUnless(isinstance(e, AssertionError))
            self.assertEqual(
                e.args,
                ("\n"
                 "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
                 "  y:2 != 3\n"
                 "  </C> != <AClass>",
                 ))
        else:
            self.fail('No exception raised!')

        self.assertEqual(c, AClass(1, 2))

        try:
            self.assertEqual(c, AClass(3, 2))
        except Exception as e:
            self.failUnless(isinstance(e, AssertionError))
            self.assertEqual(
                e.args,
                ("\n"
                 "  <C(failed):testfixtures.tests.test_comparison.AClass>\n"
                 "  x:1 != 3\n"
                 "  </C> != <AClass>",
                 ))
        else:
            self.fail('No exception raised!')

    def test_first(self):
        self.assertEqual(
            C('testfixtures.tests.sample1.SampleClassA'),
            SampleClassA()
            )

    def test_second(self):
        self.assertEqual(
            SampleClassA(),
            C('testfixtures.tests.sample1.SampleClassA'),
            )

    def test_not_same_first(self):
        self.assertNotEqual(
            C('datetime'),
            SampleClassA()
            )

    def test_not_same_second(self):
        self.assertNotEqual(
            SampleClassA(),
            C('datetime')
            )

    def test_object_supplied(self):
        self.assertEqual(
            SampleClassA(1),
            C(SampleClassA(1))
            )

    def test_class_and_vars(self):
        self.assertEqual(
            SampleClassA(1),
            C(SampleClassA, {'args': (1,)})
            )

    def test_class_and_kw(self):
        self.assertEqual(
            SampleClassA(1),
            C(SampleClassA, args=(1,))
            )

    def test_class_and_vars_and_kw(self):
        self.assertEqual(
            AClass(1, 2),
            C(AClass, {'x': 1}, y=2)
            )

    def test_object_and_vars(self):
        # vars passed are used instead of the object's
        self.assertEqual(
            SampleClassA(1),
            C(SampleClassA(), {'args': (1,)})
            )

    def test_object_and_kw(self):
        # kws passed are used instead of the object's
        self.assertEqual(
            SampleClassA(1),
            C(SampleClassA(), args=(1,))
            )

    def test_object_not_strict(self):
        # only attributes on comparison object
        # are used
        self.assertEqual(
            C(AClass(1), strict=False),
            AClass(1, 2),
            )

    def test_exception(self):
        self.assertEqual(
            ValueError('foo'),
            C(ValueError('foo'))
            )

    def test_exception_class_and_args(self):
        self.assertEqual(
            ValueError('foo'),
            C(ValueError, args=('foo', ))
            )

    def test_exception_instance_and_args(self):
        self.assertEqual(
            ValueError('foo'),
            C(ValueError('bar'), args=('foo', ))
            )

    def test_exception_not_same(self):
        self.assertNotEqual(
            ValueError('foo'),
            C(ValueError('bar'))
            )

    def test_exception_no_args_different(self):
        self.assertNotEqual(
            WeirdException(1, 2),
            C(WeirdException(1, 3))
            )

    def test_exception_no_args_same(self):
        self.assertEqual(
            C(WeirdException(1, 2)),
            WeirdException(1, 2)
            )

    def test_repr_file_different(self):
        with TempDirectory() as d:
            path = d.write('file', b'stuff')
            f = open(path)
            f.close()
        if PY3:
            c = C('io.TextIOWrapper', name=path, mode='r', closed=False,
                  strict=False)
            self.assertNotEqual(f, c)
            compare(repr(c),
                    "\n"
                    "  <C(failed):_io.TextIOWrapper>\n"
                    "  closed:False != True\n"
                    "  </C>",
                    )
        else:
            c = C(file, name=path, mode='r', closed=False, strict=False)
            self.assertNotEqual(f, c)
            compare(repr(c),
                    "\n"
                    "  <C(failed):__builtin__.file>\n"
                    "  closed:False != True\n"
                    "  </C>",
                    )

    def test_file_same(self):
        with TempDirectory() as d:
            path = d.write('file', b'stuff')
            f = open(path)
            f.close()
        if PY3:
            self.assertEqual(
                f,
                C('io.TextIOWrapper', name=path, mode='r', closed=True,
                  strict=False)
                )
        else:
            self.assertEqual(
                f,
                C(file, name=path, mode='r', closed=True, strict=False)
                )

    def test_no___dict___strict(self):
        x = X()
        try:
            self.assertEqual(
                C(X, x=1),
                x
                )
        except TypeError as e:
            self.assertEqual(e.args, (
                '<X> does not support vars() so cannot do strict comparison',
                ))
        else:
            self.fail('No exception raised!')

    def test_no___dict___not_strict_same(self):
        x = X()
        x.x = 1
        self.assertEqual(C(X, x=1, strict=False), x)

    def test_no___dict___not_strict_different(self):
        if PY3:
            expected = (
                "\n  <C(failed):testfixtures.tests.test_com[42 chars] </C>"
                " != <X>",
                )
        else:
            expected = (
                "\n"
                "  <C(failed):testfixtures.tests.test_comparison.X>\n"
                "  x:1 != 2\n"
                "  y:2 not in other\n"
                "  </C> != <X>",
                )
        x = X()
        x.x = 2
        try:
            self.assertEqual(
                C(X, x=1, y=2, strict=False),
                x
                )
        except AssertionError as e:
            compare(e.args, expected)
        else:
            self.fail('No exception raised!')

    def test_compared_object_defines_eq(self):
        # If an object defines eq, such as Django instances,
        # things become tricky

        class Annoying:
            def __init__(self):
                self.eq_called = 0

            def __eq__(self, other):
                self.eq_called += 1
                if isinstance(other, Annoying):
                    return True
                return False

        self.assertEqual(Annoying(), Annoying())

        # Suddenly, order matters.

        # This order is wrong, as it uses the class's __eq__:
        self.assertFalse(Annoying() == C(Annoying))
        if PY2:
            # although this, which is subtly different, does not:
            self.assertFalse(Annoying() != C(Annoying))
        else:
            # but on PY3 __eq__ is used as a fallback:
            self.assertTrue(Annoying() != C(Annoying))

        # This is the right ordering:
        self.assertTrue(C(Annoying) == Annoying())
        self.assertFalse(C(Annoying) != Annoying())

        # When the ordering is right, you still get the useful
        # comparison representation afterwards
        c = C(Annoying, eq_called=1)
        c == Annoying()
        self.assertEqual(
            repr(c),
            '\n  <C(failed):testfixtures.tests.test_comparison.Annoying>\n'
            '  eq_called:1 != 0\n'
            '  </C>')

    def test_compared_object_class_attributes(self):

        class Classy(object):
            x = 1
            y = 2

        self.assertEqual(C(Classy, x=1, y=2), Classy())

        c = C(Classy, x=1, y=1)
        self.assertNotEqual(c, Classy())
        self.assertEqual(
            repr(c),
            '\n  <C(failed):testfixtures.tests.test_comparison.Classy>\n'
            '  y:1 != 2\n'
            '  </C>')

        ce = C(Classy, x=1, y=1)
        ca = Classy()
        ca.y = 1
        self.assertEqual(ce, ca)

        ce = C(Classy, x=1, y=2)
        ca = Classy()
        ca.y = 1
        self.assertNotEqual(ce, ca)
        self.assertEqual(
            repr(ce),
            '\n  <C(failed):testfixtures.tests.test_comparison.Classy>\n'
            '  y:2 != 1\n'
            '  </C>')

    def test_importerror(self):
        self.failIf(
            C(ImportError('x')) != ImportError('x')
            )

    def test_class_defines_comparison_strictly(self):
        self.assertEqual(
            C('testfixtures.tests.test_comparison.FussyDefineComparison',
              attr=1),
            FussyDefineComparison(1)
            )

    def test_cant_resolve(self):
        try:
            C('testfixtures.bonkers')
        except Exception as e:
            self.failUnless(isinstance(e, AttributeError))
            self.assertEqual(
                e.args,
                ("'testfixtures.bonkers' could not be resolved", )
                )
        else:
            self.fail('No exception raised!')

    def test_no_name(self):
        class NoName(object):
            pass
        NoName.__name__ = ''
        NoName.__module__ = ''
        c = C(NoName)
        if PY3:
            expected = "<C:<class '.TestC.test_no_name.<locals>.NoName'>>"
        else:
            expected = "<C:<class '.'>>"
        self.assertEqual(repr(c), expected)



from unittest import TestCase

from types import GeneratorType

from testfixtures import generator


class TestG(TestCase):

    def test_example(self):
        g = generator(1, 2, 3)
        self.failUnless(isinstance(g, GeneratorType))
        self.assertEqual(tuple(g), (1, 2, 3))

    def test_from_sequence(self):
        s = (1, 2, 3)
        g = generator(*s)
        self.failUnless(isinstance(g, GeneratorType))
        self.assertEqual(tuple(g), (1, 2, 3))



from testfixtures import (
    log_capture, compare, Comparison as C, LogCapture, ShouldRaise
)
from unittest import TestCase

from logging import getLogger

root = getLogger()
one = getLogger('one')
two = getLogger('two')
child = getLogger('one.child')


class TestLog_Capture(TestCase):

    @log_capture('two', 'one.child')
    @log_capture('one')
    @log_capture()
    def test_logging(self, l1, l2, l3):
        # we can now log as normal
        root.info('1')
        one.info('2')
        two.info('3')
        child.info('4')
        # and later check what was logged
        l1.check(
            ('root', 'INFO', '1'),
            ('one', 'INFO', '2'),
            ('two', 'INFO', '3'),
            ('one.child', 'INFO', '4'),
            )
        l2.check(
            ('one', 'INFO', '2'),
            ('one.child', 'INFO', '4')
            )
        l3.check(
            ('two', 'INFO', '3'),
            ('one.child', 'INFO', '4')
            )
        # each logger also exposes the real
        # log records should anything else be neeeded
        compare(l3.records, [
            C('logging.LogRecord'),
            C('logging.LogRecord'),
            ])

    def test_uninstall_properly(self):
        root = getLogger()
        child = getLogger('child')
        before_root = root.handlers[:]
        before_child = child.handlers[:]
        try:
            old_root_level = root.level
            root.setLevel(49)
            old_child_level = child.level
            child.setLevel(69)

            @log_capture('child')
            @log_capture()
            def test_method(l1, l2):
                root = getLogger()
                root.info('1')
                self.assertEqual(root.level, 1)
                child = getLogger('child')
                self.assertEqual(child.level, 1)
                child.info('2')
                l1.check(
                    ('root', 'INFO', '1'),
                    ('child', 'INFO', '2'),
                    )
                l2.check(
                    ('child', 'INFO', '2'),
                    )

            test_method()

            self.assertEqual(root.level, 49)
            self.assertEqual(child.level, 69)

            self.assertEqual(root.handlers, before_root)
            self.assertEqual(child.handlers, before_child)

        finally:
            root.setLevel(old_root_level)
            child.setLevel(old_child_level)

    @log_capture()
    def test_decorator_returns_logcapture(self, l):
        # check for what we get, so we only have to write
        # tests in test_logcapture.py
        self.failUnless(isinstance(l, LogCapture))

    def test_remove_existing_handlers(self):
        logger = getLogger()
        # get original handlers
        original = logger.handlers
        try:
            # put in a stub which will blow up if used
            logger.handlers = start = [object()]

            @log_capture()
            def test_method(l):
                logger.info('during')
                l.check(('root', 'INFO', 'during'))

            test_method()

            compare(logger.handlers, start)

        finally:
            logger.handlers = original

    def test_clear_global_state(self):
        from logging import _handlers, _handlerList
        capture = LogCapture()
        capture.uninstall()
        self.assertFalse(capture in _handlers)
        self.assertFalse(capture in _handlerList)

    def test_no_propogate(self):
        logger = getLogger('child')
        # paranoid check
        compare(logger.propagate, True)

        @log_capture('child', propagate=False)
        def test_method(l):
            logger.info('a log message')
            l.check(('child', 'INFO', 'a log message'))

        with LogCapture() as global_log:
            test_method()

        global_log.check()
        compare(logger.propagate, True)

    def test_different_attributes(self):
        with LogCapture(attributes=('funcName', 'processName')) as log:
            getLogger().info('oh hai')
        log.check(
            ('test_different_attributes', 'MainProcess')
        )

    def test_missing_attribute(self):
        with LogCapture(attributes=('msg', 'lolwut')) as log:
            getLogger().info('oh %s', 'hai')
        log.check(
            ('oh %s', None)
        )

    def test_single_attribute(self):
        # one which isn't a string, to boot!
        with LogCapture(attributes=['msg']) as log:
            getLogger().info(dict(foo='bar', baz='bob'))
        log.check(
            dict(foo='bar', baz='bob'),
        )

    def test_callable_instead_of_attribute(self):
        def extract_msg(record):
            return {k: v for (k, v) in record.msg.items()
                    if k != 'baz'}
        with LogCapture(attributes=extract_msg) as log:
            getLogger().info(dict(foo='bar', baz='bob'))
        log.check(
            dict(foo='bar'),
        )

    def test_msg_is_none(self):
        with LogCapture(attributes=('msg', 'foo')) as log:
            getLogger().info(None, extra=dict(foo='bar'))
        log.check(
            (None, 'bar')
        )

    def test_normal_check(self):
        with LogCapture() as log:
            getLogger().info('oh hai')

        with ShouldRaise(AssertionError) as s:
            log.check(('root', 'INFO', 'oh noez'))

        compare(str(s.raised), expected=(
            "sequence not as expected:\n\n"
            "same:\n"
            "()\n\n"
            "expected:\n"
            "(('root', 'INFO', 'oh noez'),)\n\n"
            "actual:\n"
            "(('root', 'INFO', 'oh hai'),)"
        ))

    def test_recursive_check(self):

        with LogCapture(recursive_check=True) as log:
            getLogger().info('oh hai')

        with ShouldRaise(AssertionError) as s:
            log.check(('root', 'INFO', 'oh noez'))

        compare(str(s.raised), expected=(
            "sequence not as expected:\n\n"
            "same:\n()\n\n"
            "expected:\n(('root', 'INFO', 'oh noez'),)\n\n"
            "actual:\n(('root', 'INFO', 'oh hai'),)\n\n"
            "While comparing [0]: sequence not as expected:\n\n"
            "same:\n('root', 'INFO')\n\n"
            "expected:\n"
            "('oh noez',)\n\n"
            "actual:\n"
            "('oh hai',)\n\n"
            "While comparing [0][2]: 'oh noez' (expected) != 'oh hai' (actual)"
        ))


class BaseCaptureTest(TestCase):
    a = 33

    @log_capture()
    def test_logs_if_a_smaller_than_44(self, logs):
        logger = getLogger()
        if self.a < 44:
            logger.info('{} is smaller than 44'.format(self.a))

        logs.check(
            ('root', 'INFO', '{} is smaller than 44'.format(self.a)),
        )


class SubclassCaptureTest(BaseCaptureTest):
    a = 2



from __future__ import print_function

import sys
from unittest import TestCase

from testfixtures import OutputCapture


class TestOutputCapture(TestCase):

    def test_compare_strips(self):
        with OutputCapture() as o:
            print(' Bar! ')
        o.compare('Bar!')

    def test_stdout_and_stderr(self):
        with OutputCapture() as o:
            print('hello', file=sys.stdout)
            print('out', file=sys.stderr)
            print('there', file=sys.stdout)
            print('now', file=sys.stderr)
        o.compare("hello\nout\nthere\nnow\n")

    def test_unicode(self):
        with OutputCapture() as o:
            print(u'\u65e5', file=sys.stdout)
        o.compare(u'\u65e5\n')

    def test_separate_capture(self):
        with OutputCapture(separate=True) as o:
            print('hello', file=sys.stdout)
            print('out', file=sys.stderr)
            print('there', file=sys.stdout)
            print('now', file=sys.stderr)
        o.compare(stdout="hello\nthere\n",
                  stderr="out\nnow\n")

    def test_original_restore(self):
        o_out, o_err = sys.stdout, sys.stderr
        with OutputCapture() as o:
            self.assertFalse(sys.stdout is o_out)
            self.assertFalse(sys.stderr is o_err)
        self.assertTrue(sys.stdout is o_out)
        self.assertTrue(sys.stderr is o_err)

    def test_double_disable(self):
        o_out, o_err = sys.stdout, sys.stderr
        with OutputCapture() as o:
            self.assertFalse(sys.stdout is o_out)
            self.assertFalse(sys.stderr is o_err)
            o.disable()
            self.assertTrue(sys.stdout is o_out)
            self.assertTrue(sys.stderr is o_err)
            o.disable()
            self.assertTrue(sys.stdout is o_out)
            self.assertTrue(sys.stderr is o_err)
        self.assertTrue(sys.stdout is o_out)
        self.assertTrue(sys.stderr is o_err)

    def test_double_enable(self):
        o_out, o_err = sys.stdout, sys.stderr
        with OutputCapture() as o:
            o.disable()
            self.assertTrue(sys.stdout is o_out)
            self.assertTrue(sys.stderr is o_err)
            o.enable()
            self.assertFalse(sys.stdout is o_out)
            self.assertFalse(sys.stderr is o_err)
            o.enable()
            self.assertFalse(sys.stdout is o_out)
            self.assertFalse(sys.stderr is o_err)
        self.assertTrue(sys.stdout is o_out)
        self.assertTrue(sys.stderr is o_err)



from functools import wraps
from testfixtures import Comparison

param_docs = """

    :param exception: This can be one of the following:

                      * `None`, indicating that an exception must be
                        raised, but the type is unimportant.

                      * An exception class, indicating that the type
                        of the exception is important but not the
                        parameters it is created with.

                      * An exception instance, indicating that an
                        exception exactly matching the one supplied
                        should be raised.

    :param unless: Can be passed a boolean that, when ``True`` indicates that
                   no exception is expected. This is useful when checking
                   that exceptions are only raised on certain versions of
                   Python.
"""


class ShouldRaise(object):
    __doc__ = """
    This context manager is used to assert that an exception is raised
    within the context it is managing.
    """ + param_docs

    #: The exception captured by the context manager.
    #: Can be used to inspect specific attributes of the exception.
    raised = None

    def __init__(self, exception=None, unless=False):
        self.exception = exception
        self.expected = not unless

    def __enter__(self):
        return self

    def __exit__(self, type, actual, traceback):

        __tracebackhide__ = True
        
        # bug in python :-(
        if type is not None and not isinstance(actual, type):
            # fixed in 2.7 onwards!
            actual = type(actual)  # pragma: no cover

        self.raised = actual

        if self.expected:
            if self.exception:
                comparison = Comparison(self.exception)
                if comparison != actual:
                    repr_actual = repr(actual)
                    repr_expected = repr(self.exception)
                    message = '%s raised, %s expected' % (
                        repr_actual, repr_expected
                    )
                    if repr_actual == repr_expected:
                        extra = [', attributes differ:']
                        extra.extend(str(comparison).split('\n')[2:-1])
                        message += '\n'.join(extra)
                    raise AssertionError(message)

            elif not actual:
                raise AssertionError('No exception raised!')
        elif actual:
            raise AssertionError('%r raised, no exception expected' % actual)

        return True


class should_raise:
    __doc__ = """
    A decorator to assert that the decorated function will raised
    an exception. An exception class or exception instance may be
    passed to check more specifically exactly what exception will be
    raised.
    """ + param_docs

    def __init__(self, exception=None, unless=None):
        self.exception = exception
        self.unless = unless

    def __call__(self, target):

        @wraps(target)
        def _should_raise_wrapper(*args, **kw):
            with ShouldRaise(self.exception, self.unless):
                target(*args, **kw)

        return _should_raise_wrapper



# NB: This file is used in the documentation, if you make changes, ensure
#     you update the line numbers in popen.txt!
"""
A sample module containing the kind of code that
TestFixtures helps with testing
"""

from datetime import datetime, date


def str_now_1():
    return str(datetime.now())

now = datetime.now


def str_now_2():
    return str(now())


def str_today_1():
    return str(date.today())

today = date.today


def str_today_2():
    return str(today())

from time import time


def str_time():
    return str(time())


class X:

    def y(self):
        return "original y"

    @classmethod
    def aMethod(cls):
        return cls

    @staticmethod
    def bMethod():
        return 2


def z():
    return "original z"


class SampleClassA:
    def __init__(self, *args):
        self.args = args


class SampleClassB(SampleClassA):
    pass


def a_function():
    return (SampleClassA(1), SampleClassB(2), SampleClassA(3))

someDict = dict(
    key='value',
    complex_key=[1, 2, 3],
    )



from datetime import date
from datetime import datetime as d
from datetime import timedelta
from datetime import tzinfo
from testfixtures import test_datetime, test_date
from testfixtures import replace, Replacer, compare, ShouldRaise
from testfixtures.tests import sample1
from unittest import TestCase


class SampleTZInfo(tzinfo):

    __test__ = False

    def utcoffset(self, dt):
        return timedelta(minutes=3) + self.dst(dt)

    def dst(self, dt):
        return timedelta(minutes=1)


class SampleTZInfo2(tzinfo):

    __test__ = False

    def utcoffset(self, dt):
        return timedelta(minutes=5)

    def dst(self, dt):
        return timedelta(minutes=0)


class TestDateTime(TestCase):

    @replace('datetime.datetime', test_datetime())
    def test_now(self):
        from datetime import datetime
        compare(datetime.now(), d(2001, 1, 1, 0, 0, 0))
        compare(datetime.now(), d(2001, 1, 1, 0, 0, 10))
        compare(datetime.now(), d(2001, 1, 1, 0, 0, 30))

    @replace('datetime.datetime', test_datetime())
    def test_now_with_tz_supplied(self):
        from datetime import datetime
        info = SampleTZInfo()
        compare(datetime.now(info), d(2001, 1, 1, 0, 4, tzinfo=SampleTZInfo()))

    @replace('datetime.datetime', test_datetime(tzinfo=SampleTZInfo()))
    def test_now_with_tz_setup(self):
        from datetime import datetime
        compare(datetime.now(), d(2001, 1, 1))

    @replace('datetime.datetime', test_datetime(tzinfo=SampleTZInfo()))
    def test_now_with_tz_setup_and_supplied(self):
        from datetime import datetime
        info = SampleTZInfo2()
        compare(datetime.now(info), d(2001, 1, 1, 0, 1, tzinfo=info))

    @replace('datetime.datetime', test_datetime(tzinfo=SampleTZInfo()))
    def test_now_with_tz_setup_and_same_supplied(self):
        from datetime import datetime
        info = SampleTZInfo()
        compare(datetime.now(info), d(2001, 1, 1, tzinfo=info))

    def test_now_with_tz_instance(self):
        dt = test_datetime(d(2001, 1, 1, tzinfo=SampleTZInfo()))
        compare(dt.now(), d(2001, 1, 1))

    def test_now_with_tz_instance_and_supplied(self):
        dt = test_datetime(d(2001, 1, 1, tzinfo=SampleTZInfo()))
        info = SampleTZInfo2()
        compare(dt.now(info), d(2001, 1, 1, 0, 1, tzinfo=info))

    def test_now_with_tz_instance_and_same_supplied(self):
        dt = test_datetime(d(2001, 1, 1, tzinfo=SampleTZInfo()))
        info = SampleTZInfo()
        compare(dt.now(info), d(2001, 1, 1, tzinfo=info))

    @replace('datetime.datetime', test_datetime(2002, 1, 1, 1, 2, 3))
    def test_now_supplied(self):
        from datetime import datetime
        compare(datetime.now(), d(2002, 1, 1, 1, 2, 3))

    @replace('datetime.datetime', test_datetime(None))
    def test_now_sequence(self, t):
        t.add(2002, 1, 1, 1, 0, 0)
        t.add(2002, 1, 1, 2, 0, 0)
        t.add(2002, 1, 1, 3, 0, 0)
        from datetime import datetime
        compare(datetime.now(), d(2002, 1, 1, 1, 0, 0))
        compare(datetime.now(), d(2002, 1, 1, 2, 0, 0))
        compare(datetime.now(), d(2002, 1, 1, 3, 0, 0))

    @replace('datetime.datetime', test_datetime())
    def test_add_and_set(self, t):
        t.add(2002, 1, 1, 1, 0, 0)
        t.add(2002, 1, 1, 2, 0, 0)
        t.set(2002, 1, 1, 3, 0, 0)
        from datetime import datetime
        compare(datetime.now(), d(2002, 1, 1, 3, 0, 0))
        compare(datetime.now(), d(2002, 1, 1, 3, 0, 10))
        compare(datetime.now(), d(2002, 1, 1, 3, 0, 30))

    @replace('datetime.datetime', test_datetime(None))
    def test_add_datetime_supplied(self, t):
        from datetime import datetime
        t.add(d(2002, 1, 1, 1))
        t.add(datetime(2002, 1, 1, 2))
        compare(datetime.now(), d(2002, 1, 1, 1, 0, 0))
        compare(datetime.now(), d(2002, 1, 1, 2, 0, 0))
        tzinfo = SampleTZInfo()
        tzrepr = repr(tzinfo)
        with ShouldRaise(ValueError(
            'Cannot add datetime with tzinfo of %s as configured to use None' %(
                tzrepr
            ))):
            t.add(d(2001, 1, 1, tzinfo=tzinfo))

    def test_instantiate_with_datetime(self):
        from datetime import datetime
        t = test_datetime(datetime(2002, 1, 1, 1))
        compare(t.now(), d(2002, 1, 1, 1, 0, 0))

    @replace('datetime.datetime', test_datetime(None))
    def test_now_requested_longer_than_supplied(self, t):
        t.add(2002, 1, 1, 1, 0, 0)
        t.add(2002, 1, 1, 2, 0, 0)
        from datetime import datetime
        compare(datetime.now(), d(2002, 1, 1, 1, 0, 0))
        compare(datetime.now(), d(2002, 1, 1, 2, 0, 0))
        compare(datetime.now(), d(2002, 1, 1, 2, 0, 10))
        compare(datetime.now(), d(2002, 1, 1, 2, 0, 30))

    @replace('datetime.datetime', test_datetime(strict=True))
    def test_call(self, t):
        compare(t(2002, 1, 2, 3, 4, 5), d(2002, 1, 2, 3, 4, 5))
        from datetime import datetime
        dt = datetime(2001, 1, 1, 1, 0, 0)
        self.failIf(dt.__class__ is d)
        compare(dt, d(2001, 1, 1, 1, 0, 0))

    def test_date_return_type(self):
        with Replacer() as r:
            r.replace('datetime.datetime', test_datetime())
            from datetime import datetime
            dt = datetime(2001, 1, 1, 1, 0, 0)
            d = dt.date()
            compare(d, date(2001, 1, 1))
            self.failUnless(d.__class__ is date)

    def test_date_return_type_picky(self):
        # type checking is a bitch :-/
        date_type = test_date(strict=True)
        with Replacer() as r:
            r.replace('datetime.datetime', test_datetime(date_type=date_type,
                                                         strict=True,
                                                         ))
            from datetime import datetime
            dt = datetime(2010, 8, 26, 14, 33, 13)
            d = dt.date()
            compare(d, date_type(2010, 8, 26))
            self.failUnless(d.__class__ is date_type)

    # if you have an embedded `now` as above, *and* you need to supply
    # a list of required datetimes, then it's often simplest just to
    # do a manual try-finally with a replacer:
    def test_import_and_obtain_with_lists(self):

        t = test_datetime(None)
        t.add(2002, 1, 1, 1, 0, 0)
        t.add(2002, 1, 1, 2, 0, 0)

        from testfixtures import Replacer
        r = Replacer()
        r.replace('testfixtures.tests.sample1.now', t.now)
        try:
            compare(sample1.str_now_2(), '2002-01-01 01:00:00')
            compare(sample1.str_now_2(), '2002-01-01 02:00:00')
        finally:
            r.restore()

    @replace('datetime.datetime', test_datetime())
    def test_repr(self):
        from datetime import datetime
        compare(repr(datetime), "<class 'testfixtures.tdatetime.tdatetime'>")

    @replace('datetime.datetime', test_datetime(delta=1))
    def test_delta(self):
        from datetime import datetime
        compare(datetime.now(), d(2001, 1, 1, 0, 0, 0))
        compare(datetime.now(), d(2001, 1, 1, 0, 0, 1))
        compare(datetime.now(), d(2001, 1, 1, 0, 0, 2))

    @replace('datetime.datetime', test_datetime(delta_type='minutes'))
    def test_delta_type(self):
        from datetime import datetime
        compare(datetime.now(), d(2001, 1, 1, 0, 0, 0))
        compare(datetime.now(), d(2001, 1, 1, 0, 10, 0))
        compare(datetime.now(), d(2001, 1, 1, 0, 30, 0))

    @replace('datetime.datetime', test_datetime(None))
    def test_set(self):
        from datetime import datetime
        datetime.set(2001, 1, 1, 1, 0, 1)
        compare(datetime.now(), d(2001, 1, 1, 1, 0, 1))
        datetime.set(2002, 1, 1, 1, 0, 0)
        compare(datetime.now(), d(2002, 1, 1, 1, 0, 0))
        compare(datetime.now(), d(2002, 1, 1, 1, 0, 20))

    @replace('datetime.datetime', test_datetime(None))
    def test_set_datetime_supplied(self, t):
        from datetime import datetime
        t.set(d(2002, 1, 1, 1))
        compare(datetime.now(), d(2002, 1, 1, 1, 0, 0))
        t.set(datetime(2002, 1, 1, 2))
        compare(datetime.now(), d(2002, 1, 1, 2, 0, 0))
        tzinfo = SampleTZInfo()
        tzrepr = repr(tzinfo)
        with ShouldRaise(ValueError(
            'Cannot add datetime with tzinfo of %s as configured to use None' %(
                tzrepr
            ))):
            t.set(d(2001, 1, 1, tzinfo=tzinfo))

    @replace('datetime.datetime', test_datetime(None, tzinfo=SampleTZInfo()))
    def test_set_tz_setup(self):
        from datetime import datetime
        datetime.set(year=2002, month=1, day=1)
        compare(datetime.now(), d(2002, 1, 1))

    @replace('datetime.datetime', test_datetime(None))
    def test_set_kw(self):
        from datetime import datetime
        datetime.set(year=2002, month=1, day=1)
        compare(datetime.now(), d(2002, 1, 1))

    @replace('datetime.datetime', test_datetime(None))
    def test_set_tzinfo_kw(self):
        from datetime import datetime
        with ShouldRaise(TypeError('Cannot add using tzinfo on tdatetime')):
            datetime.set(year=2002, month=1, day=1, tzinfo=SampleTZInfo())

    @replace('datetime.datetime', test_datetime(None))
    def test_set_tzinfo_args(self):
        from datetime import datetime
        with ShouldRaise(TypeError('Cannot add using tzinfo on tdatetime')):
            datetime.set(2002, 1, 2, 3, 4, 5, 6, SampleTZInfo())

    @replace('datetime.datetime', test_datetime(None))
    def test_add_kw(self, t):
        from datetime import datetime
        t.add(year=2002, day=1, month=1)
        compare(datetime.now(), d(2002, 1, 1))

    @replace('datetime.datetime', test_datetime(None))
    def test_add_tzinfo_kw(self, t):
        from datetime import datetime
        with ShouldRaise(TypeError('Cannot add using tzinfo on tdatetime')):
            datetime.add(year=2002, month=1, day=1, tzinfo=SampleTZInfo())

    @replace('datetime.datetime', test_datetime(None))
    def test_add_tzinfo_args(self, t):
        from datetime import datetime
        with ShouldRaise(TypeError('Cannot add using tzinfo on tdatetime')):
            datetime.add(2002, 1, 2, 3, 4, 5, 6, SampleTZInfo())

    @replace('datetime.datetime',
             test_datetime(2001, 1, 2, 3, 4, 5, 6, SampleTZInfo()))
    def test_max_number_args(self):
        from datetime import datetime
        compare(datetime.now(), d(2001, 1, 2, 3, 4, 5, 6))

    @replace('datetime.datetime', test_datetime(2001, 1, 2))
    def test_min_number_args(self):
        from datetime import datetime
        compare(datetime.now(), d(2001, 1, 2))

    @replace('datetime.datetime', test_datetime(
        year=2001,
        month=1,
        day=2,
        hour=3,
        minute=4,
        second=5,
        microsecond=6,
        tzinfo=SampleTZInfo()
        ))
    def test_all_kw(self):
        from datetime import datetime
        compare(datetime.now(), d(2001, 1, 2, 3, 4, 5, 6))

    @replace('datetime.datetime', test_datetime(2001, 1, 2))
    def test_utc_now(self):
        from datetime import datetime
        compare(datetime.utcnow(), d(2001, 1, 2))

    @replace('datetime.datetime',
             test_datetime(2001, 1, 2, tzinfo=SampleTZInfo()))
    def test_utc_now_with_tz(self):
        from datetime import datetime
        compare(datetime.utcnow(), d(2001, 1, 1, 23, 56))

    @replace('datetime.datetime', test_datetime(strict=True))
    def test_isinstance_strict(self):
        from datetime import datetime
        to_check = []
        to_check.append(datetime(1999, 1, 1))
        to_check.append(datetime.now())
        to_check.append(datetime.now(SampleTZInfo()))
        to_check.append(datetime.utcnow())
        datetime.set(2001, 1, 1, 20)
        to_check.append(datetime.now())
        datetime.add(2001, 1, 1, 21)
        to_check.append(datetime.now())
        to_check.append(datetime.now())
        datetime.set(datetime(2001, 1, 1, 22))
        to_check.append(datetime.now())
        to_check.append(datetime.now(SampleTZInfo()))
        datetime.add(datetime(2001, 1, 1, 23))
        to_check.append(datetime.now())
        to_check.append(datetime.now())
        to_check.append(datetime.now(SampleTZInfo()))
        datetime.set(d(2001, 1, 1, 22))
        to_check.append(datetime.now())
        datetime.add(d(2001, 1, 1, 23))
        to_check.append(datetime.now())
        to_check.append(datetime.now())
        to_check.append(datetime.now(SampleTZInfo()))

        for inst in to_check:
            self.failUnless(isinstance(inst, datetime), inst)
            self.failUnless(inst.__class__ is datetime, inst)
            self.failUnless(isinstance(inst, d), inst)
            self.failIf(inst.__class__ is d, inst)

    @replace('datetime.datetime', test_datetime())
    def test_isinstance_default(self):
        from datetime import datetime
        to_check = []
        to_check.append(datetime(1999, 1, 1))
        to_check.append(datetime.now())
        to_check.append(datetime.now(SampleTZInfo()))
        to_check.append(datetime.utcnow())
        datetime.set(2001, 1, 1, 20)
        to_check.append(datetime.now())
        datetime.add(2001, 1, 1, 21)
        to_check.append(datetime.now())
        to_check.append(datetime.now(SampleTZInfo()))
        datetime.set(datetime(2001, 1, 1, 22))
        to_check.append(datetime.now())
        datetime.add(datetime(2001, 1, 1, 23))
        to_check.append(datetime.now())
        to_check.append(datetime.now())
        to_check.append(datetime.now(SampleTZInfo()))
        datetime.set(d(2001, 1, 1, 22))
        to_check.append(datetime.now())
        datetime.add(d(2001, 1, 1, 23))
        to_check.append(datetime.now())
        to_check.append(datetime.now())
        to_check.append(datetime.now(SampleTZInfo()))

        for inst in to_check:
            self.failIf(isinstance(inst, datetime), inst)
            self.failIf(inst.__class__ is datetime, inst)
            self.failUnless(isinstance(inst, d), inst)
            self.failUnless(inst.__class__ is d, inst)

    def test_subsecond_deltas(self):
        datetime = test_datetime(delta=0.5)
        compare(datetime.now(), datetime(2001, 1, 1, 0, 0, 0, 0))
        compare(datetime.now(), datetime(2001, 1, 1, 0, 0, 0, 500000))
        compare(datetime.now(), datetime(2001, 1, 1, 0, 0, 1, 0))

    def test_ms_delta(self):
        datetime = test_datetime(delta=100, delta_type='microseconds')
        compare(datetime.now(), datetime(2001, 1, 1, 0, 0, 0, 0))
        compare(datetime.now(), datetime(2001, 1, 1, 0, 0, 0, 100))
        compare(datetime.now(), datetime(2001, 1, 1, 0, 0, 0, 200))

    def test_tick_when_static(self):
        datetime = test_datetime(delta=0)
        compare(datetime.now(), expected=d(2001, 1, 1))
        datetime.tick(hours=1)
        compare(datetime.now(), expected=d(2001, 1, 1, 1))

    def test_tick_when_dynamic(self):
        # hopefully not that common?
        datetime = test_datetime()
        compare(datetime.now(), expected=d(2001, 1, 1))
        datetime.tick(hours=1)
        compare(datetime.now(), expected=d(2001, 1, 1, 1, 0, 10))

    def test_tick_with_timedelta_instance(self):
        datetime = test_datetime(delta=0)
        compare(datetime.now(), expected=d(2001, 1, 1))
        datetime.tick(timedelta(hours=1))
        compare(datetime.now(), expected=d(2001, 1, 1, 1))



SECRET_KEY = 'fake-key'
INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    "testfixtures.tests.test_django",
]

DATABASES = {'default': {'ENGINE': 'django.db.backends.sqlite3'}}
